#
# echodbg is used to echo debug messages to stderr
#
proc echodbg args {
  global DEBUG
  if { "$DEBUG" == "true" } {
    puts stderr "$args"
  }
}

#
# echoerr is used to echo messages to stderr
#
proc echoerr str {
  global NOEXIT
  puts stderr "$str"
  exit 1
}

#
# echoout is used to echo messages to stdout
#
proc echoout str {
  puts stdout "$str"
}

#
# posix2nt converts posix style filenames to NT style filenames
#
proc posix2nt {src} {
  regsub -all / $src \\ result
  return $result
}

#
# execit is used to execute commands and capture their output. The output is
# returned in the 'output' argument and the procedure returns with the exit code
# of the command.
#
proc execit {output args} {
  upvar $output result
  set exitval [catch {eval exec $args} result]
  return $exitval
}

#
# execute_command is used to execute a command and capture errors.
#   opt - 1: display output, 0: don't display output
#
proc execute_command {opt args} {
  set exitval [catch {eval exec $args} result]
  if {$opt == 1} {
    puts stdout "$result"
  }
  return $exitval
}

#
# filegrep searches for 'str' in file 'f'. The search is case insensitive.
# Returns 1 if found, 0 if not found
#
proc filegrep {f str} {
  set retval 0
  set in [open $f r]
  while {[gets $in line] >= 0} {
    if {[regexp -nocase $str $line]} {
      set retval 1
      break
    }
  }
  close $in
  return $retval
}

proc getcalendars {} {
  global FILE MAEHOME

  execute_command 0 $MAEHOME\/bin\/composer "create $FILE from CAL"
}

proc checkcal {} {
  global FILE CALENDAR

  if {[filegrep $FILE $CALENDAR]} {
    echoerr "Calendar $CALENDAR already exists"
  }
}

proc checkhol {} {
  global FILE CALTYPE

  if {[filegrep $FILE HOLIDAYS] == 0} {
    echoerr "Type $CALTYPE calendars require a HOLIDAYS calendar to be pre-defined"
  }
}

proc checkstartdate {} {
  global STARTDATE CALTYPE

  if {"$STARTDATE" == ""} {
    echoerr "Type $CALTYPE calendars require a -s parameter"
  }
}
###########################################################################
# PMR 63130 02/11/99
# Validated the inputs for various options. They are -r, -d, -w, -f.
# Also replaced fatalerr function call with echoerr. There was no function
# called fatalerr before. So the script used to fail to display any errors.
###########################################################################

proc parse_args {} {
  global FILE CALENDAR CALTYPE CALDESC CALARG ITERATIONS STARTDATE OUTPUT
  global COMPILE argc argv DEBUG

  for {set x 0} {$x<$argc} {incr x 1} {
    set opt [lindex $argv $x]
    set val [lindex $argv [expr $x+1]]
    switch -exact -- $opt -debug {
      set DEBUG "true"
      echodbg "Setting DEBUG to true"
    } -c {
      set CALENDAR $val
      echodbg "Setting CALENDAR to $val"
    } -d {
      if {"$val" == ""} {
        echoerr "Error: expected argument with -d"
        exit 1
      }
      set CALTYPE "D"
      set CALDESC "\"Day $val of every month\""
      if {($val > 31) || ($val < 1)} {
         echoerr "Error: Day $val not in range 1-31"
      }
      set CALARG $val
      incr x 1
      echodbg "Setting CALTYPE to D"
    } -e {
      set CALTYPE "E"
      set CALDESC "\"End of month calendar\""
      echodbg "Setting CALTYPE to E"
    } -i {
      if {"$val" == ""} {
        echoerr "Error: expected argument with -i"
        exit 1
      }
      set ITERATIONS $val
      incr x 1
      echodbg "Setting ITERATIONS to $val"
    } -f {
      if {"$val" == ""} {
        echoerr "Error: expected argument with -f"
        exit 1
      }
      set CALTYPE "F"
      set CALDESC "\"Fiscal month-end calendar\""
      if {($val > 3) || ($val < 1)} {
         echoerr "Error: Argument $val not in range 1-3"
      }
      set CALARG $val
      incr x 1
      echodbg "Setting CALTYPE to F"
    } -l {
      set CALTYPE "L"
      set CALDESC "\"Last workday of month\""
      checkhol
      echodbg "Setting CALTYPE to L"
    } -m {
      set CALTYPE "M"
      set CALDESC "\"Semi-monthly calendar\""
      echodbg "Setting CALTYPE to M"
    } -p {
      if {"$val" == ""} {
        echoerr "Error: expected argument with -p"
        exit 1
      }
      set CALTYPE "P"
      set CALDESC "\"Workday before day $val\""
      if {($val > 31) || ($val < 1)} {
         echoerr "Error: Day $val not in range 1-31"
      }
      set CALARG $val
      checkhol
      incr x 1
      echodbg "Setting CALTYPE to P"
    } -s {
      if {"$val" == ""} {
        echoerr "Error: expected argument with -s"
        exit 1
      }
      set STARTDATE $val
      incr x 1
      echodbg "Setting STARTDATE to $val"
    } -r {
      if {"$val" == ""} {
        echoerr "Error: expected argument with -r"
        exit 1
      }
      if {($val > 31) || ($val < 1)} {
         echoerr "Error: Day $val not in range 1-31"
      }
      set CALTYPE "R"
      set CALDESC "\"Every $val days\""
      set CALARG $val
      incr x 1
      echodbg "Setting CALTYPE to R"
    } -w {
      if {"$val" == ""} {
        echoerr "Error: expected argument with -w"
        exit 1
      }
      set CALTYPE "W"
      set CALDESC "\"Workday after day $val every month\""
      if {($val > 31) || ($val < 1)} {
         echoerr "Error: Day $val not in range 1-31"
      }
      set CALARG $val
      checkhol
      incr x 1
      echodbg "Setting CALTYPE to W"
    } -x {
      set OUTPUT "stdout"
      echodbg "Setting OUTPUT to stdout"
    } -z {
      set COMPILE "yes"
      echodbg "Setting COMPILE to yes"
    } 
  }
}

#
# Start of main script
#
#############################################################################
# PMR 63130 02/11/99
# Now that the PATH is not modified in makecal.cmd we need to give absolute
# path for the composer and datecalc calls. To know why the PATH is not 
# modified, see the comments in makecal.cmd
#############################################################################

set PATCHLEVEL "\$Revision: 1.3.1.4 $"
set DEBUG "false"
set COMPILE "no"
set OUTPUT "composer"
set STARTDATE ""
set CALTYPE ""
set CALARG ""
set CALDESC ""
set MAEHOME $env(POSIXHOME)
set FILE "/temp/makecal.[pid]"
execit CALENDAR $MAEHOME\/bin\/datecalc today pic HHTT
set CALENDAR C$CALENDAR
set ITERATIONS 12
execit THISMONTH $MAEHOME\/bin\/datecalc today pic MM
set THISMONTH [expr [string trimleft $THISMONTH 0] - 1]

# PR fix 5801. Get all the 4 digits of the year for 2000
# dates to work...
execit THISYEAR $MAEHOME\/bin\/datecalc today pic YYYY
set THISYEAR [string trimleft $THISYEAR 0]
set months(0) "DEC"
set months(1) "JAN"
set months(2) "FEB"
set months(3) "MAR"
set months(4) "APR"
set months(5) "MAY"
set months(6) "JUN"
set months(7) "JUL"
set months(8) "AUG"
set months(9) "SEP"
set months(10) "OCT"
set months(11) "NOV"
set months(12) "DEC"
set monthend(0) 31
set monthend(1) 31
set monthend(2) 28
set monthend(3) 31
set monthend(4) 30
set monthend(5) 31
set monthend(6) 30
set monthend(7) 31
set monthend(8) 31
set monthend(9) 30
set monthend(10) 31
set monthend(11) 30
set monthend(12) 31

if {$argc == 1} {
   if {[lindex $argv 0] == "-h"} { 
       puts "Valid options are:"
       puts "	-c name	Make a calendar with the given name"
       puts "	-e	Last day of every month"
       puts "	-d n	nth day of every month"
       puts "	-f n	fiscal calendar, 1 = 445, 2 = 454, 3 = 544 (2)" 
       puts "	-h 	help"
       puts "	-i n	put dates from i months starting this month into calendar"
       puts "	-l	Last workday of the month (1)"
       puts "	-m	1st and 15th of every month"
       puts "	-p n	Previous workday before nth of the month (1)"
       puts "	-r n	Every nth day (2)"
       puts "	-s n	Start date for -r, -f options, must be a valid"
       puts "		DATECALC date (ie. \"JAN 10 1995\" not JAN 10 1995)"
       puts "	-u	print command usage"
       puts "	-v	print version information"
       puts "	-w n	Next workday after nth of the month (1)"
       puts "	-x	Don't add to composer, output to stdout"
       puts "	-z	Compile into Symphony  WARNING: This option"
       puts "		will re-submit schedules from the current"
       puts "		day's production.  Jobs/Schedules may need"
       puts "		to be cancelled."
       puts "	-U	print command usage"
       puts "	-V	print version information"
       puts " NOTES: (1) Requires a HOLIDAYS calendar already exist in Symphony"
       puts "        (2) Requires an starting date or calendar name to"
       puts "            be specified with the -s option"
       exit 0
  }
}
getcalendars
parse_args
checkcal
if {"$CALTYPE" == "R"} {
  checkstartdate
}
if {"$CALTYPE" == "F"} {
  checkstartdate
  set ITERATIONS [expr $ITERATIONS / 3]
}

if {"$OUTPUT" == "stdout"} {
  set f stdout
} else {
  set f [open $FILE a]
}

puts $f "$CALENDAR $CALDESC"

set loop 1
set month 1
set year $THISYEAR
set numdays 1
set offset $CALARG

while {$loop <= $ITERATIONS} {
  set currmon [expr $loop + $THISMONTH]
  set month [expr $currmon % 12]
  if {$month == 0} {
    set month 12
  }
  set eom $monthend($month)
  # PR 5926 (07/08/98)
  # The order of evaluation was wrong. So for year 2000
  # end of month was set to 29 for all months.
  # So added box-brackets accordingly.
  #
  if {($month == 2) && ((([expr $year % 4] == 0) && ([expr $year % 100] != 0)) || ([expr $year % 400] == 0))} {
    set eom 29
  }
  switch -exact -- $CALTYPE D {
    if {$month == 2} {
      if {$CALARG > $eom} {
        set offset $eom
      } else {
        set offset $CALARG
      }
    } else {
      if {$CALARG > $monthend($month)} {
        set offset $monthend($month)
      } else {
        set offset $CALARG
      }
    }
    puts -nonewline $f " $month/$offset/$year"
  } E {
    puts -nonewline $f " $month/$eom/$year"
  } F {
    set month [expr $loop - 1]
    set offset [expr $month * 13]
    switch -exact -- $CALARG 1 {
      set numdays [expr $offset + 4]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
      set numdays [expr $offset + 4 + 4]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
      set numdays [expr $offset + 4 + 4 + 5]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
    } 2 {
      set numdays [expr $offset + 4]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
      set numdays [expr $offset + 4 + 5]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
      set numdays [expr $offset + 4 + 5 + 4]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
    } 3 {
      set numdays [expr $offset + 5]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
      set numdays [expr $offset + 5 + 4]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
      set numdays [expr $offset + 5 + 4 + 4]
      execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays weeks pic mm/dd/yy
      puts -nonewline $f " $line"
    }
  } L {
    execit line $MAEHOME\/bin\/datecalc $months($month) 1 $year - 1 workdays pic mm/dd/yy
    puts -nonewline $f " $line"
  } M {
    puts -nonewline $f " $month/1/$year"
    puts -nonewline $f " $month/15/$year"
  } P {
###########################################################################
# PMR 63130 02/11/99
# Before passing the date to datecalc get a valid end of month.
###########################################################################
    if {$CALARG > $monthend($month)} {
        set newoffset $monthend($month)
    } else {
        set newoffset $CALARG
    }
    if {($month == 2) && ((([expr $year % 4] == 0) && ([expr $year % 100] != 0)) || ([expr $year % 400] == 0))} {
        set newoffset 29
    }
    execit line $MAEHOME\/bin\/datecalc $months($month) $newoffset $year - 1 workdays pic mm/dd/yy
    puts -nonewline $f " $line"
  } R {
    set numdays [expr $loop * $offset]
    execit line $MAEHOME\/bin\/datecalc $STARTDATE + $numdays days pic mm/dd/yy
    puts -nonewline $f " $line"
  } W {
    execit line $MAEHOME\/bin\/datecalc $months($month) $CALARG $year + 1 workdays pic mm/dd/yy
    puts -nonewline $f " $line"
  }

  if {[expr $loop % 8] == 0} {
    puts $f " "
  }
  if {$month == 12} {
    set year [expr $year + 1]

#   PR 5801 makecal -l -i 21 gave Bad date specification error...
#   We can go beyond 2000. 
#    if {$year > 99} {
#      set year [expr $year - 100]
#    }

  }
  set loop [expr $loop + 1]
}
puts $f " "
if {"$OUTPUT" != "stdout"} {
  close $f
}

if {"$OUTPUT" == "composer"} {
  execute_command 1 $MAEHOME\/bin\/composer \"rep $FILE\"
}

#########################################################################
# PMR 63130 
# We need to cleanup the temp directory. So took out the conditional
# cleaning.
#if {"$OUTPUT" != "stdout"} 
#########################################################################
file delete [posix2nt $FILE]


if {"$COMPILE" == "no"} {
  if {"$OUTPUT" == "composer"} {
    echoout "New calendar will be available next production day"
  }
} else {
  echoout "WARNING: New production day will take effect now ..."
  execit DATE $MAEHOME\/bin\/datecalc today pic YYYYMMDDHHTT
  execute_command 0 $MAEHOME\/bin\/conman "link @;noask"
  execute_command 0 $MAEHOME\/bin\/schedulr -autodate
  execute_command 0 $MAEHOME\/bin\/compiler
  execute_command 0 $MAEHOME\/bin\/conman "stop @;wait;noask"
  execute_command 0 $MAEHOME\/bin\/stageman -log=M$DATE
  execute_command 0 $MAEHOME\/bin\/conman start
}


exit 0
